@@ -1,5 +1,5 @@
 use miette::{
-    DefaultReportPrinter, Diagnostic, DiagnosticReport, MietteError, MietteTheme,
+    Diagnostic, DiagnosticReport, GraphicalReportPrinter, GraphicalTheme, MietteError, NamedSource,
     NarratableReportPrinter, SourceSpan,
 };
 use thiserror::Error;
@@ -8,7 +8,7 @@ fn fmt_report(diag: DiagnosticReport) -> String {
     let mut out = String::new();
     // Mostly for dev purposes.
     if std::env::var("STYLE").is_ok() {
-        DefaultReportPrinter::new_themed(MietteTheme::unicode())
+        GraphicalReportPrinter::new_themed(GraphicalTheme::unicode())
             .render_report(&mut out, diag.inner())
             .unwrap();
     } else if std::env::var("NARRATED").is_ok() {
@@ -16,7 +16,7 @@ fn fmt_report(diag: DiagnosticReport) -> String {
             .render_report(&mut out, diag.inner())
             .unwrap();
     } else {
-        DefaultReportPrinter::new_themed(MietteTheme::unicode_nocolor())
+        GraphicalReportPrinter::new_themed(GraphicalTheme::unicode_nocolor())
             .render_report(&mut out, diag.inner())
             .unwrap();
     };
@@ -29,19 +29,19 @@ fn single_line_highlight() -> Result<(), MietteError> {
     #[error("oops!")]
     #[diagnostic(code(oops::my::bad), help("try doing it better next time?"))]
     struct MyBad {
-        src: String,
-        #[snippet(src, "This is the part that broke")]
+        src: NamedSource,
+        #[snippet(src, message("This is the part that broke"))]
         ctx: SourceSpan,
-        #[highlight(ctx)]
+        #[highlight(ctx, label = "this bit here")]
         highlight: SourceSpan,
     }
 
     let src = "source\n  text\n    here".to_string();
     let len = src.len();
     let err = MyBad {
-        src,
-        ctx: ("bad_file.rs", 0, len).into(),
-        highlight: ("this bit here", 9, 4).into(),
+        src: NamedSource::new("bad_file.rs", src),
+        ctx: (0, len).into(),
+        highlight: (9, 4).into(),
     };
     let out = fmt_report(err.into());
     println!("{}", out);
@@ -65,14 +65,56 @@ fn single_line_highlight() -> Result<(), MietteError> {
     Ok(())
 }
 
+#[test]
+fn single_line_highlight_with_empty_span() -> Result<(), MietteError> {
+    #[derive(Debug, Diagnostic, Error)]
+    #[error("oops!")]
+    #[diagnostic(code(oops::my::bad), help("try doing it better next time?"))]
+    struct MyBad {
+        src: NamedSource,
+        #[snippet(src, message("This is the part that broke"))]
+        ctx: SourceSpan,
+        #[highlight(ctx, label = "this bit here")]
+        highlight: SourceSpan,
+    }
+
+    let src = "source\n  text\n    here".to_string();
+    let len = src.len();
+    let err = MyBad {
+        src: NamedSource::new("bad_file.rs", src),
+        ctx: (0, len).into(),
+        highlight: (9, 0).into(),
+    };
+    let out = fmt_report(err.into());
+    println!("{}", out);
+    let expected = r#"
+────[oops::my::bad]────────────────────
+
+    × oops!
+
+   ╭───[bad_file.rs:1:1] This is the part that broke:
+ 1 │ source
+ 2 │   text
+   ·   ┬
+   ·   ╰─ this bit here
+ 3 │     here
+
+    ‽ try doing it better next time?
+"#
+    .trim_start()
+    .to_string();
+    assert_eq!(expected, out);
+    Ok(())
+}
+
 #[test]
 fn single_line_highlight_no_label() -> Result<(), MietteError> {
     #[derive(Debug, Diagnostic, Error)]
     #[error("oops!")]
     #[diagnostic(code(oops::my::bad), help("try doing it better next time?"))]
     struct MyBad {
-        src: String,
-        #[snippet(src, "This is the part that broke")]
+        src: NamedSource,
+        #[snippet(src, message("This is the part that broke"))]
         ctx: SourceSpan,
         #[highlight(ctx)]
         highlight: SourceSpan,
@@ -81,8 +123,8 @@ fn single_line_highlight_no_label() -> Result<(), MietteError> {
     let src = "source\n  text\n    here".to_string();
     let len = src.len();
     let err = MyBad {
-        src,
-        ctx: ("bad_file.rs", 0, len).into(),
+        src: NamedSource::new("bad_file.rs", src),
+        ctx: (0, len).into(),
         highlight: (9, 4).into(),
     };
     let out = fmt_report(err.into());
@@ -112,22 +154,22 @@ fn multiple_same_line_highlights() -> Result<(), MietteError> {
     #[error("oops!")]
     #[diagnostic(code(oops::my::bad), help("try doing it better next time?"))]
     struct MyBad {
-        src: String,
-        #[snippet(src, "This is the part that broke")]
+        src: NamedSource,
+        #[snippet(src, message("This is the part that broke"))]
         ctx: SourceSpan,
-        #[highlight(ctx)]
+        #[highlight(ctx, label = "this bit here")]
         highlight1: SourceSpan,
-        #[highlight(ctx)]
+        #[highlight(ctx, label = "also this bit")]
         highlight2: SourceSpan,
     }
 
     let src = "source\n  text text text text text\n    here".to_string();
     let len = src.len();
     let err = MyBad {
-        src,
-        ctx: ("bad_file.rs", 0, len).into(),
-        highlight1: ("this bit here", 9, 4).into(),
-        highlight2: ("also this bit", 14, 4).into(),
+        src: NamedSource::new("bad_file.rs", src),
+        ctx: (0, len).into(),
+        highlight1: (9, 4).into(),
+        highlight2: (14, 4).into(),
     };
     let out = fmt_report(err.into());
     println!("{}", out);
@@ -158,19 +200,19 @@ fn multiline_highlight_adjacent() -> Result<(), MietteError> {
     #[error("oops!")]
     #[diagnostic(code(oops::my::bad), help("try doing it better next time?"))]
     struct MyBad {
-        src: String,
-        #[snippet(src, "This is the part that broke")]
+        src: NamedSource,
+        #[snippet(src, message("This is the part that broke"))]
         ctx: SourceSpan,
-        #[highlight(ctx)]
+        #[highlight(ctx, label = "these two lines")]
         highlight: SourceSpan,
     }
 
     let src = "source\n  text\n    here".to_string();
     let len = src.len();
     let err = MyBad {
-        src,
-        ctx: ("bad_file.rs", 0, len).into(),
-        highlight: ("these two lines", 9, 11).into(),
+        src: NamedSource::new("bad_file.rs", src),
+        ctx: (0, len).into(),
+        highlight: (9, 11).into(),
     };
     let out = fmt_report(err.into());
     println!("{}", out);
@@ -199,12 +241,12 @@ fn multiline_highlight_flyby() -> Result<(), MietteError> {
     #[error("oops!")]
     #[diagnostic(code(oops::my::bad), help("try doing it better next time?"))]
     struct MyBad {
-        src: String,
-        #[snippet(src, "This is the part that broke")]
+        src: NamedSource,
+        #[snippet(src, message("This is the part that broke"))]
         ctx: SourceSpan,
-        #[highlight(ctx)]
+        #[highlight(ctx, label = "block 1")]
         highlight1: SourceSpan,
-        #[highlight(ctx)]
+        #[highlight(ctx, label = "block 2")]
         highlight2: SourceSpan,
     }
 
@@ -217,10 +259,10 @@ line5
     .to_string();
     let len = src.len();
     let err = MyBad {
-        src,
-        ctx: ("bad_file.rs", 0, len).into(),
-        highlight1: ("block 1", 0, len).into(),
-        highlight2: ("block 2", 10, 9).into(),
+        src: NamedSource::new("bad_file.rs", src),
+        ctx: (0, len).into(),
+        highlight1: (0, len).into(),
+        highlight2: (10, 9).into(),
     };
     let out = fmt_report(err.into());
     println!("{}", out);
@@ -252,10 +294,10 @@ fn multiline_highlight_no_label() -> Result<(), MietteError> {
     #[error("oops!")]
     #[diagnostic(code(oops::my::bad), help("try doing it better next time?"))]
     struct MyBad {
-        src: String,
-        #[snippet(src, "This is the part that broke")]
+        src: NamedSource,
+        #[snippet(src, message("This is the part that broke"))]
         ctx: SourceSpan,
-        #[highlight(ctx)]
+        #[highlight(ctx, label = "block 1")]
         highlight1: SourceSpan,
         #[highlight(ctx)]
         highlight2: SourceSpan,
@@ -270,9 +312,9 @@ line5
     .to_string();
     let len = src.len();
     let err = MyBad {
-        src,
-        ctx: ("bad_file.rs", 0, len).into(),
-        highlight1: ("block 1", 0, len).into(),
+        src: NamedSource::new("bad_file.rs", src),
+        ctx: (0, len).into(),
+        highlight1: (0, len).into(),
         highlight2: (10, 9).into(),
     };
     let out = fmt_report(err.into());
@@ -304,22 +346,22 @@ fn multiple_multiline_highlights_adjacent() -> Result<(), MietteError> {
     #[error("oops!")]
     #[diagnostic(code(oops::my::bad), help("try doing it better next time?"))]
     struct MyBad {
-        src: String,
-        #[snippet(src, "This is the part that broke")]
+        src: NamedSource,
+        #[snippet(src, message("This is the part that broke"))]
         ctx: SourceSpan,
-        #[highlight(ctx)]
+        #[highlight(ctx, label = "this bit here")]
         highlight1: SourceSpan,
-        #[highlight(ctx)]
+        #[highlight(ctx, label = "also this bit")]
         highlight2: SourceSpan,
     }
 
     let src = "source\n  text\n    here\nmore here".to_string();
     let len = src.len();
     let err = MyBad {
-        src,
-        ctx: ("bad_file.rs", 0, len).into(),
-        highlight1: ("this bit here", 0, 10).into(),
-        highlight2: ("also this bit", 20, 6).into(),
+        src: NamedSource::new("bad_file.rs", src),
+        ctx: (0, len).into(),
+        highlight1: (0, 10).into(),
+        highlight2: (20, 6).into(),
     };
     let out = fmt_report(err.into());
     println!("{}", out);
@@ -354,22 +396,22 @@ fn multiple_multiline_highlights_overlapping_lines() -> Result<(), MietteError>
     #[error("oops!")]
     #[diagnostic(code(oops::my::bad), help("try doing it better next time?"))]
     struct MyBad {
-        src: String,
-        #[snippet(src, "This is the part that broke")]
+        src: NamedSource,
+        #[snippet(src, message("This is the part that broke"))]
         ctx: SourceSpan,
-        #[highlight(ctx)]
+        #[highlight(ctx, label = "this bit here")]
         highlight1: SourceSpan,
-        #[highlight(ctx)]
+        #[highlight(ctx, label = "also this bit")]
         highlight2: SourceSpan,
     }
 
     let src = "source\n  text\n    here".to_string();
     let len = src.len();
     let err = MyBad {
-        src,
-        ctx: ("bad_file.rs", 0, len).into(),
-        highlight1: ("this bit here", 0, 8).into(),
-        highlight2: ("also this bit", 9, 10).into(),
+        src: NamedSource::new("bad_file.rs", src),
+        ctx: (0, len).into(),
+        highlight1: (0, 8).into(),
+        highlight2: (9, 10).into(),
     };
     let out = fmt_report(err.into());
     println!("{}", out);
@@ -385,25 +427,67 @@ fn multiple_multiline_highlights_overlapping_offsets() -> Result<(), MietteError
     #[error("oops!")]
     #[diagnostic(code(oops::my::bad), help("try doing it better next time?"))]
     struct MyBad {
-        src: String,
-        #[snippet(src, "This is the part that broke")]
+        src: NamedSource,
+        #[snippet(src, message("This is the part that broke"))]
         ctx: SourceSpan,
-        #[highlight(ctx)]
+        #[highlight(ctx, label = "this bit here")]
         highlight1: SourceSpan,
-        #[highlight(ctx)]
+        #[highlight(ctx, label = "also this bit")]
         highlight2: SourceSpan,
     }
 
     let src = "source\n  text\n    here".to_string();
     let len = src.len();
     let err = MyBad {
-        src,
-        ctx: ("bad_file.rs", 0, len).into(),
-        highlight1: ("this bit here", 0, 8).into(),
-        highlight2: ("also this bit", 10, 10).into(),
+        src: NamedSource::new("bad_file.rs", src),
+        ctx: (0, len).into(),
+        highlight1: (0, 8).into(),
+        highlight2: (10, 10).into(),
     };
     let out = fmt_report(err.into());
     println!("{}", out);
     assert_eq!("Error [oops::my::bad]: oops!\n\n[bad_file.rs] This is the part that broke:\n\n 1 │ source\n 2 │   text\n   ·   ──┬─\n   ·     ╰── this bit here\n 3 │     here\n\n﹦ try doing it better next time?\n".to_string(), out);
     Ok(())
 }
+
+#[test]
+fn url_links() -> Result<(), MietteError> {
+    #[derive(Debug, Diagnostic, Error)]
+    #[error("oops!")]
+    #[diagnostic(
+        code(oops::my::bad),
+        help("try doing it better next time?"),
+        url("https://example.com")
+    )]
+    struct MyBad;
+    let err = MyBad;
+    let out = fmt_report(err.into());
+    println!("{}", out);
+    assert!(out.contains("https://example.com"));
+    assert!(out.contains("click for details"));
+    assert!(out.contains("oops::my::bad"));
+    Ok(())
+}
+
+#[test]
+fn disable_url_links() -> Result<(), MietteError> {
+    #[derive(Debug, Diagnostic, Error)]
+    #[error("oops!")]
+    #[diagnostic(
+        code(oops::my::bad),
+        help("try doing it better next time?"),
+        url("https://example.com")
+    )]
+    struct MyBad;
+    let err = MyBad;
+    let mut out = String::new();
+    GraphicalReportPrinter::new_themed(GraphicalTheme::unicode_nocolor())
+        .without_code_linking()
+        .render_report(&mut out, &err)
+        .unwrap();
+    println!("{}", out);
+    assert!(!out.contains("https://example.com"));
+    assert!(!out.contains("click for details"));
+    assert!(out.contains("oops::my::bad"));
+    Ok(())
+}
